@@ -65,6 +65,18 @@ impl Keybindings {
         self.bindings.get(&key_combo).cloned()
     }
 
+    /// Remove a keybinding
+    ///
+    /// Returns `Some(ReedlineEvent)` if the keycombination was previously bound to a particular [`ReedlineEvent`]
+    pub fn remove_binding(
+        &mut self,
+        modifier: KeyModifiers,
+        key_code: KeyCode,
+    ) -> Option<ReedlineEvent> {
+        let key_combo = KeyCombination { modifier, key_code };
+        self.bindings.remove(&key_combo)
+    }
+
     /// Get assigned keybindings
     pub fn get_keybindings(&self) -> &HashMap<KeyCombination, ReedlineEvent> {
         &self.bindings
@@ -83,7 +95,14 @@ pub fn add_common_keybindings(kb: &mut Keybindings) {
     kb.add_binding(KM::NONE, KC::Esc, ReedlineEvent::Esc);
     kb.add_binding(KM::NONE, KC::Backspace, edit_bind(EC::Backspace));
     kb.add_binding(KM::NONE, KC::Delete, edit_bind(EC::Delete));
-    kb.add_binding(KM::NONE, KC::End, edit_bind(EC::MoveToLineEnd));
+    kb.add_binding(
+        KM::NONE,
+        KC::End,
+        ReedlineEvent::UntilFound(vec![
+            ReedlineEvent::HistoryHintComplete,
+            edit_bind(EC::MoveToLineEnd),
+        ]),
+    );
     kb.add_binding(KM::NONE, KC::Home, edit_bind(EC::MoveToLineStart));
 
     kb.add_binding(KM::CONTROL, KC::Char('c'), ReedlineEvent::CtrlC);
@@ -99,6 +118,8 @@ pub fn add_common_keybindings(kb: &mut Keybindings) {
         ]),
     );
     kb.add_binding(KM::CONTROL, KC::Left, edit_bind(EC::MoveWordLeft));
+    kb.add_binding(KM::CONTROL, KC::Backspace, edit_bind(EC::BackspaceWord));
+    kb.add_binding(KM::CONTROL, KC::Delete, edit_bind(EC::DeleteWord));
 
     kb.add_binding(
         KM::NONE,
