@@ -1,5 +1,3 @@
-use crate::{menu::IndexDirection, utils::text_manipulation};
-
 use {
     crate::{
         completion::{CircularCompletionHandler, Completer, DefaultCompleter},
@@ -7,13 +5,14 @@ use {
         edit_mode::{EditMode, Emacs},
         enums::{EventStatus, ReedlineEvent},
         highlighter::SimpleMatchHighlighter,
-        hinter::{DefaultHinter, Hinter},
+        hinter::Hinter,
         history::{FileBackedHistory, History, HistoryNavigationQuery},
-        menu::{parse_selection_char, Menu, MenuEvent},
+        menu::{Menu, MenuEvent, ReedlineMenu},
         painting::{Painter, PromptLines},
         prompt::{PromptEditMode, PromptHistorySearchStatus},
-        DefaultValidator, EditCommand, ExampleHighlighter, Highlighter, Prompt,
-        PromptHistorySearch, Signal, ValidationResult, Validator,
+        utils::text_manipulation,
+        EditCommand, ExampleHighlighter, Highlighter, Prompt, PromptHistorySearch, Signal,
+        ValidationResult, Validator,
     },
     crossterm::{
         event,
@@ -23,6 +22,9 @@ use {
     std::{borrow::Borrow, io, time::Duration},
 };
 
+#[cfg(feature = "bashisms")]
+use crate::menu_functions::{parse_selection_char, ParseAction};
+
 // The POLL_WAIT is used to specify for how long the POLL should wait for
 // events, to accelerate the handling of paste or compound resize events. Having
 // a POLL_WAIT of zero means that every single event is treated as soon as it
@@ -57,9 +59,8 @@ enum InputMode {
 ///
 /// ## Example usage
 /// ```no_run
-/// use std::io;
 /// use reedline::{Reedline, Signal, DefaultPrompt};
-/// let mut line_editor = Reedline::create()?;
+/// let mut line_editor = Reedline::create();
 /// let prompt = DefaultPrompt::default();
 ///
 /// let out = line_editor.read_line(&prompt).unwrap();
@@ -72,7 +73,6 @@ enum InputMode {
 ///
 ///    }
 /// }
-/// # Ok::<(), io::Error>(())
 /// ```
 pub struct Reedline {
     editor: Editor,
@@ -82,7 +82,7 @@ pub struct Reedline {
     input_mode: InputMode,
 
     // Validator
-    validator: Box<dyn Validator>,
+    validator: Option<Box<dyn Validator>>,
 
     // Stdout
     painter: Painter,
@@ -102,7 +102,7 @@ pub struct Reedline {
     highlighter: Box<dyn Highlighter>,
 
     // Showcase hints based on various strategies (history, language-completion, spellcheck, etc)
-    hinter: Box<dyn Hinter>,
+    hinter: Option<Box<dyn Hinter>>,
     hide_hints: bool,
 
     // Is Some(n) read_line() should repaint prompt every `n` milliseconds
@@ -112,7 +112,7 @@ pub struct Reedline {
     use_ansi_coloring: bool,
 
     // Engine Menus
-    menus: Vec<Box<dyn Menu>>,
+    menus: Vec<ReedlineMenu>,
 }
 
 impl Drop for Reedline {
@@ -125,16 +125,17 @@ impl Drop for Reedline {
 
 impl Reedline {
     /// Create a new [`Reedline`] engine with a local [`History`] that is not synchronized to a file.
-    pub fn create() -> io::Result<Reedline> {
+    #[must_use]
+    pub fn create() -> Self {
         let history = Box::new(FileBackedHistory::default());
         let painter = Painter::new(std::io::BufWriter::new(std::io::stderr()));
         let buffer_highlighter = Box::new(ExampleHighlighter::default());
         let completer = Box::new(DefaultCompleter::default());
-        let hinter = Box::new(DefaultHinter::default());
-        let validator = Box::new(DefaultValidator);
+        let hinter = None;
+        let validator = None;
         let edit_mode = Box::new(Emacs::default());
 
-        let reedline = Reedline {
+        Reedline {
             editor: Editor::default(),
             history,
             input_mode: InputMode::Regular,
@@ -151,9 +152,7 @@ impl Reedline {
             animate: false,
             use_ansi_coloring: true,
             menus: Vec::new(),
-        };
-
-        Ok(reedline)
+        }
     }
 
     /// A builder to include a [`Hinter`] in your instance of the Reedline engine
@@ -162,21 +161,26 @@ impl Reedline {
     /// //Cargo.toml
     /// //[dependencies]
     /// //nu-ansi-term = "*"
-    /// use std::io;
     /// use {
     ///     nu_ansi_term::{Color, Style},
     ///     reedline::{DefaultHinter, Reedline},
     /// };
     ///
-    /// let mut line_editor = Reedline::create()?.with_hinter(Box::new(
+    /// let mut line_editor = Reedline::create().with_hinter(Box::new(
     ///     DefaultHinter::default()
     ///     .with_style(Style::new().italic().fg(Color::LightGray)),
     /// ));
-    /// # Ok::<(), io::Error>(())
     /// ```
     #[must_use]
     pub fn with_hinter(mut self, hinter: Box<dyn Hinter>) -> Self {
-        self.hinter = hinter;
+        self.hinter = Some(hinter);
+        self
+    }
+
+    /// Remove current [`Hinter`]
+    #[must_use]
+    pub fn disable_hints(mut self) -> Self {
+        self.hinter = None;
         self
     }
 
@@ -185,7 +189,6 @@ impl Reedline {
     /// ```rust,no_run
     /// // Create a reedline object with tab completions support
     ///
-    /// use std::io;
     /// use reedline::{DefaultCompleter, Reedline};
     ///
     /// let commands = vec![
@@ -196,8 +199,7 @@ impl Reedline {
     /// ];
     /// let completer = Box::new(DefaultCompleter::new_with_wordlen(commands.clone(), 2));
     ///
-    /// let mut line_editor = Reedline::create()?.with_completer(completer);
-    /// # Ok::<(), io::Error>(())
+    /// let mut line_editor = Reedline::create().with_completer(completer);
     /// ```
     #[must_use]
     pub fn with_completer(mut self, completer: Box<dyn Completer>) -> Self {
@@ -242,7 +244,6 @@ impl Reedline {
     /// ```rust,no_run
     /// // Create a reedline object with highlighter support
     ///
-    /// use std::io;
     /// use reedline::{ExampleHighlighter, Reedline};
     ///
     /// let commands = vec![
@@ -252,8 +253,7 @@ impl Reedline {
     ///   "this is the reedline crate".into(),
     /// ];
     /// let mut line_editor =
-    /// Reedline::create()?.with_highlighter(Box::new(ExampleHighlighter::new(commands)));
-    /// # Ok::<(), io::Error>(())
+    /// Reedline::create().with_highlighter(Box::new(ExampleHighlighter::new(commands)));
     /// ```
     #[must_use]
     pub fn with_highlighter(mut self, highlighter: Box<dyn Highlighter>) -> Self {
@@ -266,22 +266,19 @@ impl Reedline {
     /// ```rust,no_run
     /// // Create a reedline object with history support, including history size limits
     ///
-    /// use std::io;
     /// use reedline::{FileBackedHistory, Reedline};
     ///
     /// let history = Box::new(
     /// FileBackedHistory::with_file(5, "history.txt".into())
     ///     .expect("Error configuring history with file"),
     /// );
-    /// let mut line_editor = Reedline::create()?
-    ///     .with_history(history)
-    ///     .expect("Error configuring reedline with history");
-    /// # Ok::<(), io::Error>(())
+    /// let mut line_editor = Reedline::create()
+    ///     .with_history(history);
     /// ```
-    pub fn with_history(mut self, history: Box<dyn History>) -> std::io::Result<Reedline> {
+    #[must_use]
+    pub fn with_history(mut self, history: Box<dyn History>) -> Self {
         self.history = history;
-
-        Ok(self)
+        self
     }
 
     /// A builder that configures the validator for your instance of the Reedline engine
@@ -289,16 +286,21 @@ impl Reedline {
     /// ```rust,no_run
     /// // Create a reedline object with validator support
     ///
-    /// use std::io;
     /// use reedline::{DefaultValidator, Reedline};
     ///
     /// let mut line_editor =
-    /// Reedline::create()?.with_validator(Box::new(DefaultValidator));
-    /// # Ok::<(), io::Error>(())
+    /// Reedline::create().with_validator(Box::new(DefaultValidator));
     /// ```
     #[must_use]
     pub fn with_validator(mut self, validator: Box<dyn Validator>) -> Self {
-        self.validator = validator;
+        self.validator = Some(validator);
+        self
+    }
+
+    /// Remove the current [`Validator`]
+    #[must_use]
+    pub fn disable_validator(mut self) -> Self {
+        self.validator = None;
         self
     }
 
@@ -311,11 +313,18 @@ impl Reedline {
 
     /// A builder that appends a menu to the engine
     #[must_use]
-    pub fn with_menu(mut self, menu: Box<dyn Menu>) -> Self {
+    pub fn with_menu(mut self, menu: ReedlineMenu) -> Self {
         self.menus.push(menu);
         self
     }
 
+    /// A builder that clears the list of menus added to the engine
+    #[must_use]
+    pub fn clear_menus(mut self) -> Self {
+        self.menus = Vec::new();
+        self
+    }
+
     /// Returns the corresponding expected prompt style for the given edit mode
     pub fn prompt_edit_mode(&self) -> PromptEditMode {
         self.edit_mode.edit_mode()
@@ -336,6 +345,17 @@ impl Reedline {
         Ok(())
     }
 
+    /// Read-only view of the history
+    pub fn history(&self) -> &dyn History {
+        &*self.history
+    }
+
+    /// Update the underlying [`History`] to/from disk
+    pub fn sync_history(&mut self) -> std::io::Result<()> {
+        // TODO: check for interactions in the non-submitting events
+        self.history.sync()
+    }
+
     /// Wait for input and provide the user with a specified [`Prompt`].
     ///
     /// Returns a [`crossterm::Result`] in which the `Err` type is [`crossterm::ErrorKind`]
@@ -542,13 +562,16 @@ impl Reedline {
                 }
                 Ok(EventStatus::Handled)
             }
+            ReedlineEvent::Esc => {
+                self.input_mode = InputMode::Regular;
+                Ok(EventStatus::Handled)
+            }
             // TODO: Check if events should be handled
             ReedlineEvent::Right
             | ReedlineEvent::Left
             | ReedlineEvent::ActionHandler
             | ReedlineEvent::Multiple(_)
             | ReedlineEvent::None
-            | ReedlineEvent::Esc
             | ReedlineEvent::HistoryHintWordComplete
             | ReedlineEvent::Menu(_)
             | ReedlineEvent::MenuNext
@@ -573,11 +596,11 @@ impl Reedline {
                     if let Some(menu) = self.menus.iter_mut().find(|menu| menu.name() == name) {
                         menu.menu_event(MenuEvent::Activate(self.quick_completions));
 
-                        if self.quick_completions {
+                        if self.quick_completions && menu.can_quick_complete() {
                             menu.update_values(
                                 self.editor.line_buffer(),
+                                self.completer.as_mut(),
                                 self.history.as_ref(),
-                                self.completer.as_ref(),
                             );
 
                             if menu.get_values().len() == 1 {
@@ -589,8 +612,8 @@ impl Reedline {
                             && menu.can_partially_complete(
                                 self.quick_completions,
                                 self.editor.line_buffer(),
+                                self.completer.as_mut(),
                                 self.history.as_ref(),
-                                self.completer.as_ref(),
                             )
                         {
                             return Ok(EventStatus::Handled);
@@ -658,35 +681,37 @@ impl Reedline {
                     })
             }
             ReedlineEvent::HistoryHintComplete => {
-                let current_hint = self.hinter.complete_hint();
-                if self.hints_active()
-                    && self.editor.is_cursor_at_buffer_end()
-                    && !current_hint.is_empty()
-                    && self.active_menu().is_none()
-                {
-                    self.run_edit_commands(&[EditCommand::InsertString(current_hint)]);
-                    Ok(EventStatus::Handled)
-                } else {
-                    Ok(EventStatus::Inapplicable)
+                if let Some(hinter) = self.hinter.as_mut() {
+                    let current_hint = hinter.complete_hint();
+                    if self.hints_active()
+                        && self.editor.is_cursor_at_buffer_end()
+                        && !current_hint.is_empty()
+                        && self.active_menu().is_none()
+                    {
+                        self.run_edit_commands(&[EditCommand::InsertString(current_hint)]);
+                        return Ok(EventStatus::Handled);
+                    }
                 }
+                Ok(EventStatus::Inapplicable)
             }
             ReedlineEvent::HistoryHintWordComplete => {
-                let current_hint_part = self.hinter.next_hint_token();
-                if self.hints_active()
-                    && self.editor.is_cursor_at_buffer_end()
-                    && !current_hint_part.is_empty()
-                    && self.active_menu().is_none()
-                {
-                    self.run_edit_commands(&[EditCommand::InsertString(current_hint_part)]);
-                    Ok(EventStatus::Handled)
-                } else {
-                    Ok(EventStatus::Inapplicable)
+                if let Some(hinter) = self.hinter.as_mut() {
+                    let current_hint_part = hinter.next_hint_token();
+                    if self.hints_active()
+                        && self.editor.is_cursor_at_buffer_end()
+                        && !current_hint_part.is_empty()
+                        && self.active_menu().is_none()
+                    {
+                        self.run_edit_commands(&[EditCommand::InsertString(current_hint_part)]);
+                        return Ok(EventStatus::Handled);
+                    }
                 }
+                Ok(EventStatus::Inapplicable)
             }
             ReedlineEvent::ActionHandler => {
                 let line_buffer = self.editor.line_buffer();
                 self.circular_completion_handler
-                    .handle(self.completer.as_ref(), line_buffer);
+                    .handle(self.completer.as_mut(), line_buffer);
                 Ok(EventStatus::Handled)
             }
             ReedlineEvent::Esc => {
@@ -728,28 +753,32 @@ impl Reedline {
                     }
                 }
 
+                #[cfg(feature = "bashisms")]
                 if let Some(event) = self.parse_bang_command() {
                     return self.handle_editor_event(prompt, event);
                 }
 
                 let buffer = self.editor.get_buffer().to_string();
-                if matches!(self.validator.validate(&buffer), ValidationResult::Complete) {
-                    self.hide_hints = true;
-                    // Additional repaint to show the content without hints etc.
-                    self.repaint(prompt)?;
-                    self.history.append(self.editor.get_buffer());
-                    self.run_edit_commands(&[EditCommand::Clear]);
-                    self.editor.reset_undo_stack();
+                match self.validator.as_mut().map(|v| v.validate(&buffer)) {
+                    None | Some(ValidationResult::Complete) => {
+                        self.hide_hints = true;
+                        // Additional repaint to show the content without hints etc.
+                        self.repaint(prompt)?;
+                        self.history.append(self.editor.get_buffer());
+                        self.run_edit_commands(&[EditCommand::Clear]);
+                        self.editor.reset_undo_stack();
 
-                    Ok(EventStatus::Exits(Signal::Success(buffer)))
-                } else {
-                    #[cfg(windows)]
-                    {
-                        self.run_edit_commands(&[EditCommand::InsertChar('\r')]);
+                        Ok(EventStatus::Exits(Signal::Success(buffer)))
                     }
-                    self.run_edit_commands(&[EditCommand::InsertChar('\n')]);
+                    Some(ValidationResult::Incomplete) => {
+                        #[cfg(windows)]
+                        {
+                            self.run_edit_commands(&[EditCommand::InsertChar('\r')]);
+                        }
+                        self.run_edit_commands(&[EditCommand::InsertChar('\n')]);
 
-                    Ok(EventStatus::Handled)
+                        Ok(EventStatus::Handled)
+                    }
                 }
             }
             ReedlineEvent::ExecuteHostCommand(host_command) => {
@@ -759,12 +788,12 @@ impl Reedline {
             ReedlineEvent::Edit(commands) => {
                 self.run_edit_commands(&commands);
                 if let Some(menu) = self.menus.iter_mut().find(|men| men.is_active()) {
-                    if self.quick_completions {
+                    if self.quick_completions && menu.can_quick_complete() {
                         menu.menu_event(MenuEvent::Edit(self.quick_completions));
                         menu.update_values(
                             self.editor.line_buffer(),
+                            self.completer.as_mut(),
                             self.history.as_ref(),
-                            self.completer.as_ref(),
                         );
 
                         if menu.get_values().len() == 1 {
@@ -781,7 +810,6 @@ impl Reedline {
 
                 Ok(EventStatus::Handled)
             }
-            ReedlineEvent::Mouse => Ok(EventStatus::Inapplicable),
             ReedlineEvent::Resize(width, height) => {
                 self.painter.handle_resize(width, height);
                 Ok(EventStatus::Handled)
@@ -856,12 +884,12 @@ impl Reedline {
                 // Exhausting the event handlers is still considered handled
                 Ok(EventStatus::Inapplicable)
             }
-            ReedlineEvent::None => Ok(EventStatus::Inapplicable),
+            ReedlineEvent::None | ReedlineEvent::Mouse => Ok(EventStatus::Inapplicable),
         }
     }
 
-    fn active_menu(&mut self) -> Option<&mut Box<dyn Menu>> {
-        self.menus.iter_mut().find(|men| men.is_active())
+    fn active_menu(&mut self) -> Option<&mut ReedlineMenu> {
+        self.menus.iter_mut().find(|menu| menu.is_active())
     }
 
     fn previous_history(&mut self) {
@@ -1041,6 +1069,7 @@ impl Reedline {
         }
     }
 
+    #[cfg(feature = "bashisms")]
     /// Parses the ! command to replace entries from the history
     fn parse_bang_command(&mut self) -> Option<ReedlineEvent> {
         let buffer = self.editor.get_buffer();
@@ -1055,19 +1084,25 @@ impl Reedline {
         let history_result = parsed
             .index
             .zip(parsed.marker)
-            .and_then(|(index, indicator)| {
-                if matches!(parsed.direction, IndexDirection::Backward) {
-                    self.history
-                        .iter_chronologic()
-                        .rev()
-                        .nth(index.saturating_sub(1))
-                        .map(|history| (parsed.remainder.len(), indicator.len(), history.clone()))
-                } else {
-                    self.history
-                        .iter_chronologic()
-                        .nth(index)
-                        .map(|history| (parsed.remainder.len(), indicator.len(), history.clone()))
-                }
+            .and_then(|(index, indicator)| match parsed.action {
+                ParseAction::BackwardSearch => self
+                    .history
+                    .iter_chronologic()
+                    .rev()
+                    .nth(index.saturating_sub(1))
+                    .map(|history| (parsed.remainder.len(), indicator.len(), history.clone())),
+                ParseAction::ForwardSearch => self
+                    .history
+                    .iter_chronologic()
+                    .nth(index)
+                    .map(|history| (parsed.remainder.len(), indicator.len(), history.clone())),
+                ParseAction::LastToken => self
+                    .history
+                    .iter_chronologic()
+                    .rev()
+                    .next()
+                    .and_then(|history| history.split_whitespace().rev().next())
+                    .map(|token| (parsed.remainder.len(), indicator.len(), token.to_string())),
             });
 
         if let Some((start, size, history)) = history_result {
@@ -1142,12 +1177,14 @@ impl Reedline {
             );
 
         let hint: String = if self.hints_active() {
-            self.hinter.handle(
-                buffer_to_paint,
-                cursor_position_in_buffer,
-                self.history.as_ref(),
-                self.use_ansi_coloring,
-            )
+            self.hinter.as_mut().map_or_else(String::new, |hinter| {
+                hinter.handle(
+                    buffer_to_paint,
+                    cursor_position_in_buffer,
+                    self.history.as_ref(),
+                    self.use_ansi_coloring,
+                )
+            })
         } else {
             String::new()
         };
@@ -1169,18 +1206,14 @@ impl Reedline {
             if menu.is_active() {
                 menu.update_working_details(
                     self.editor.line_buffer(),
+                    self.completer.as_mut(),
                     self.history.as_ref(),
-                    self.completer.as_ref(),
                     &self.painter,
                 );
             }
         }
 
-        let menu = self
-            .menus
-            .iter()
-            .find(|menu| menu.is_active())
-            .map(|menu| menu.as_ref());
+        let menu = self.menus.iter().find(|menu| menu.is_active());
 
         self.painter
             .repaint_buffer(prompt, &lines, menu, self.use_ansi_coloring)
@@ -1190,5 +1223,5 @@ impl Reedline {
 #[test]
 fn thread_safe() {
     fn f<S: Send>(_: S) {}
-    f(Reedline::create().unwrap());
+    f(Reedline::create());
 }
