@@ -13,10 +13,9 @@
 //! ```rust,no_run
 //! // Create a default reedline object to handle user input
 //!
-//! # use std::io;
 //! use reedline::{DefaultPrompt, Reedline, Signal};
 //!
-//!  let mut line_editor = Reedline::create()?;
+//!  let mut line_editor = Reedline::create();
 //!  let prompt = DefaultPrompt::default();
 //!
 //!  loop {
@@ -37,7 +36,6 @@
 //!          }
 //!      }
 //!  }
-//! # Ok::<(), io::Error>(())
 //! ```
 //! ## Integrate with custom keybindings
 //!
@@ -48,7 +46,6 @@
 //! //    [dependencies]
 //! //    crossterm = "*"
 //!
-//! # use std::io;
 //! use {
 //!   crossterm::event::{KeyCode, KeyModifiers},
 //!   reedline::{default_emacs_keybindings, EditCommand, Reedline, Emacs, ReedlineEvent},
@@ -62,8 +59,7 @@
 //! );
 //! let edit_mode = Box::new(Emacs::new(keybindings));
 //!
-//! let mut line_editor = Reedline::create()?.with_edit_mode(edit_mode);
-//! # Ok::<(), io::Error>(())
+//! let mut line_editor = Reedline::create().with_edit_mode(edit_mode);
 //! ```
 //!
 //! ## Integrate with [`History`]
@@ -71,17 +67,14 @@
 //! ```rust,no_run
 //! // Create a reedline object with history support, including history size limits
 //!
-//! # use std::io;
 //! use reedline::{FileBackedHistory, Reedline};
 //!
 //! let history = Box::new(
 //!     FileBackedHistory::with_file(5, "history.txt".into())
 //!         .expect("Error configuring history with file"),
 //! );
-//! let mut line_editor = Reedline::create()?
-//!     .with_history(history)
-//!     .expect("Error configuring reedline with history");
-//! # Ok::<(), io::Error>(())
+//! let mut line_editor = Reedline::create()
+//!     .with_history(history);
 //! ```
 //!
 //! ## Integrate with custom syntax [`Highlighter`]
@@ -89,7 +82,6 @@
 //! ```rust,no_run
 //! // Create a reedline object with highlighter support
 //!
-//! # use std::io;
 //! use reedline::{ExampleHighlighter, Reedline};
 //!
 //! let commands = vec![
@@ -99,8 +91,7 @@
 //!   "this is the reedline crate".into(),
 //! ];
 //! let mut line_editor =
-//! Reedline::create()?.with_highlighter(Box::new(ExampleHighlighter::new(commands)));
-//! # Ok::<(), io::Error>(())
+//! Reedline::create().with_highlighter(Box::new(ExampleHighlighter::new(commands)));
 //! ```
 //!
 //! ## Integrate with custom tab completion
@@ -108,8 +99,7 @@
 //! ```rust,no_run
 //! // Create a reedline object with tab completions support
 //!
-//! # use std::io;
-//! use reedline::{DefaultCompleter, Reedline, CompletionMenu};
+//! use reedline::{ColumnarMenu, DefaultCompleter, Reedline, ReedlineMenu};
 //!
 //! let commands = vec![
 //!   "test".into(),
@@ -119,10 +109,10 @@
 //! ];
 //! let completer = Box::new(DefaultCompleter::new_with_wordlen(commands.clone(), 2));
 //! // Use the interactive menu to select options from the completer
-//! let completion_menu = Box::new(CompletionMenu::default());
+//! let completion_menu = Box::new(ColumnarMenu::default().with_name("completion_menu"));
 //!
-//! let mut line_editor = Reedline::create()?.with_completer(completer).with_menu(completion_menu);
-//! # Ok::<(), io::Error>(())
+//! let mut line_editor =
+//! Reedline::create().with_completer(completer).with_menu(ReedlineMenu::EngineCompleter(completion_menu));
 //! ```
 //!
 //! ## Integrate with [`Hinter`] for fish-style history autosuggestions
@@ -134,18 +124,16 @@
 //! //    [dependencies]
 //! //    nu-ansi-term = "*"
 //!
-//! # use std::io;
 //! use {
 //!   nu_ansi_term::{Color, Style},
 //!   reedline::{DefaultHinter, Reedline},
 //! };
 //!
 //!
-//! let mut line_editor = Reedline::create()?.with_hinter(Box::new(
+//! let mut line_editor = Reedline::create().with_hinter(Box::new(
 //!   DefaultHinter::default()
 //!   .with_style(Style::new().italic().fg(Color::LightGray)),
 //! ));
-//! # Ok::<(), io::Error>(())
 //! ```
 //!
 //! ## Are we prompt yet? (Development status)
@@ -177,7 +165,7 @@
 //!
 //! ### Development history
 //!
-//! If you want to follow along with the history how reedline got started, you can watch the [recordings](https://youtube.com/playlist?list=PLP2yfE2-FXdQw0I6O4YdIX_mzBeF5TDdv) of [JT](https://github.com/jntrnr)`s [live-coding streams](https://www.twitch.tv/jntrnr).
+//! If you want to follow along with the history how reedline got started, you can watch the [recordings](https://youtube.com/playlist?list=PLP2yfE2-FXdQw0I6O4YdIX_mzBeF5TDdv) of [JT](https://github.com/jntrnr)'s [live-coding streams](https://www.twitch.tv/jntrnr).
 //!
 //! [Playlist: Creating a line editor in Rust](https://youtube.com/playlist?list=PLP2yfE2-FXdQw0I6O4YdIX_mzBeF5TDdv)
 //!
@@ -222,7 +210,7 @@ mod highlighter;
 pub use highlighter::{ExampleHighlighter, Highlighter, SimpleMatchHighlighter};
 
 mod completion;
-pub use completion::{Completer, DefaultCompleter, Span};
+pub use completion::{Completer, DefaultCompleter, Span, Suggestion};
 
 mod hinter;
 pub use hinter::{DefaultHinter, Hinter};
@@ -231,7 +219,9 @@ mod validator;
 pub use validator::{DefaultValidator, ValidationResult, Validator};
 
 mod menu;
-pub use menu::{CompletionMenu, HistoryMenu, Menu, MenuEvent};
+pub use menu::{
+    menu_functions, ColumnarMenu, ListMenu, Menu, MenuEvent, MenuTextStyle, ReedlineMenu,
+};
 
 mod utils;
 pub use utils::{
