@@ -1,10 +1,13 @@
-use super::utils::{coerce_crlf, line_width};
-
 use {
-    crate::{menu::Menu, painting::PromptLines, Prompt},
+    super::utils::{coerce_crlf, line_width},
+    crate::{
+        menu::{Menu, ReedlineMenu},
+        painting::PromptLines,
+        Prompt,
+    },
     crossterm::{
         cursor::{self, MoveTo, RestorePosition, SavePosition},
-        style::{Print, ResetColor, SetForegroundColor},
+        style::{Attribute, Print, ResetColor, SetAttribute, SetForegroundColor},
         terminal::{self, Clear, ClearType, ScrollUp},
         QueueableCommand, Result,
     },
@@ -73,7 +76,8 @@ impl Painter {
         self.terminal_size.0
     }
 
-    fn remaining_lines(&self) -> u16 {
+    /// Returns the available lines from the prompt down
+    pub fn remaining_lines(&self) -> u16 {
         self.screen_height() - self.prompt_start_row
     }
 
@@ -124,7 +128,7 @@ impl Painter {
         &mut self,
         prompt: &dyn Prompt,
         lines: &PromptLines,
-        menu: Option<&dyn Menu>,
+        menu: Option<&ReedlineMenu>,
         use_ansi_coloring: bool,
     ) -> Result<()> {
         self.stdout.queue(cursor::Hide)?;
@@ -219,13 +223,14 @@ impl Painter {
         &mut self,
         prompt: &dyn Prompt,
         lines: &PromptLines,
-        menu: Option<&dyn Menu>,
+        menu: Option<&ReedlineMenu>,
         use_ansi_coloring: bool,
     ) -> Result<()> {
         // print our prompt with color
         if use_ansi_coloring {
             self.stdout
-                .queue(SetForegroundColor(prompt.get_prompt_color()))?;
+                .queue(SetForegroundColor(prompt.get_prompt_color()))?
+                .queue(SetAttribute(Attribute::Bold))?;
         }
 
         self.stdout
@@ -235,8 +240,21 @@ impl Painter {
             Some(menu) => menu.indicator(),
             None => &lines.prompt_indicator,
         };
+
+        if use_ansi_coloring {
+            self.stdout
+                .queue(SetForegroundColor(prompt.get_indicator_color()))?
+                .queue(SetAttribute(Attribute::Bold))?;
+        }
+
         self.stdout.queue(Print(&coerce_crlf(prompt_indicator)))?;
 
+        if use_ansi_coloring {
+            self.stdout
+                .queue(SetForegroundColor(prompt.get_prompt_right_color()))?
+                .queue(SetAttribute(Attribute::Bold))?;
+        }
+
         self.print_right_prompt(lines)?;
 
         if use_ansi_coloring {
@@ -261,7 +279,7 @@ impl Painter {
         &mut self,
         prompt: &dyn Prompt,
         lines: &PromptLines,
-        menu: Option<&dyn Menu>,
+        menu: Option<&ReedlineMenu>,
         use_ansi_coloring: bool,
     ) -> Result<()> {
         let screen_width = self.screen_width();
